from wr340g import Tplink_WR340G, Tplink_WR340Gv4

from wr720n import Tplink_WR720N

from wr740n import Tplink_WR740N
from wr741n import Tplink_WR741N

from wr940n import Tplink_WR940N
from wr941n import Tplink_WR941N
